/******************************************************************************/
/******************************************************************************/
/***                                                                        ***/
/***                        main.c : LED Desk Light                         ***/
/***                                                                        ***/
/*** 쐬 : 2024.05.^                                    by : S.Suzuki   ***/
/******************************************************************************/
/******************************************************************************/
/*  : ATtiny85płB                                                    */

#include	<avr/io.h>
//#include	<avr/wdt.h>
#include	<avr/interrupt.h>
#include	"main.h"
#include	"ADConv.h"

/******************************************************************************/
/*                 ^C}PgC^[o^C}                         */
/******************************************************************************/

#define TMR1_10mS	156				/* OCR1CɃZbgl10mSi10mS / (1 / 15.625KHz) = 156.25j  */

static volatile BYTE	s_byTmWait10m;


/* Wrbg^C}P̏ ***************************************************/

static void Init_Timer1(void)
{
	/* ^C}P䃌WX^ */
#if PCLK == CLK01			/* PCLK=1MHẑƂ */
	TCCR1 = 0x87;					/*    b7:CTC1  = 1 : OCR1CƈvŃ^C}NA     */
									/*    b6:PWM1A = 0 : PWMAȂ                  */
									/* b5-b4:COM1A[1-0] = 00 : OC1A[qsgp          */
									/* b3-b0:CS1[3-0] = 0111 : ^C}NbNPCLK/64 */
									/*                  PCLK=1MHẑƂ 15.625KHz    */
#elif PCLK == CLK08			/* PCLK=8MHẑƂ */
	TCCR1 = 0x8A;					/*    b7:CTC1  = 1 : OCR1CƈvŃ^C}NA        */
									/*    b6:PWM1A = 0 : PWMAȂ                     */
									/* b5-b4:COM1A1`COM1A0 = 00 : OC1A[qsgp         */
									/* b3-b0:CS13`CS10 = 1010 : ^C}NbNPCLK/512 */
									/*                    PCLK=8MHẑƂ 15.625KHz     */
#elif
	#error Peripheral Clock Setting Error.
#endif
	/* ʃ^C}䃌WX^iӁF^C}Oƃ^C}P̋pj */
	GTCCR = GTCCR & 0x81;			/*    b7:TSM   = X : ^C}/JE^[h? */
									/*    b6:PWM1B = 0 : PWMBȂ               */
									/* b5-b4:COM1B1`COM1B0 = 00 : OC1B[qsgp   */
									/*    b3:FOC1B = 0 : OC1BύXȂ           */
									/*    b2:FOC1A = 0 : OC1AύXȂ           */
									/*    b1:PSR1  = 0 : vXP1̃Zbg  */
									/*    b0:PSR0  = X : vXP0̃ZbgHH  */
	/* 荞݋WX^iӁF^C}Oƃ^C}P̋pj */
	TIMSK = (TIMSK & 0x9B) | 0x40;	/*    b7:irbgȂj                         */
									/*    b6:OCIE1A = 1 : OCR1AƈvŊ荞݋ */
									/*    b5:OCIE1B = 0 : OCR1BƈvŊ荞݋֎~ */
									/*    b4:OCIE0A = X : OCR0AƈvŊ荞݁HH */
									/*    b3:OCIE0B = X : OCR0BƈvŊ荞݁HH */
									/*    b2:TOIE1  = 0 : TCNT1I[o[t[荞݋֎~ */
									/*    b1:TOIE0  = X : TCNT0I[o[t[荞݁HH */
									/*    b0:irbgȂj                         */
	/* 荞ݗvtOWX^iӁF^C}Oƃ^C}P̋pj */
	TIFR = 0x64;					/*    b7:irbgȂj                                       */
									/*    b6:OCF1A = 1 : OCF1ANAiOCR1AƈvŊ荞ݗvj */
									/*    b5:OCF1B = 1 : OCF1BNAiOCR1BƈvŊ荞ݗvj */
									/*    b4:OCF0A = 0 :        iOCR0AƈvŊ荞ݗvj */
									/*    b3:OCF0B = 0 :        iOCR0BƈvŊ荞ݗvj */
									/*    b2:TOV1  = 1 : TOV1NAiTCNT1I[o[t[荞ݗvj */
									/*    b1:TOV0  = 0 :       iTCNT0I[o[t[荞ݗvj */
									/*    b0:irbgȂj                                       */
	/* PLL^Xe[^XWX^ */
	PLLCSR = 0x00;					/*    b7:LSM  = 0 : ᑬ얳                   */
									/* b6-b3:irbgȂj                            */
									/*    b2:PCKE = 0 : PCK֎~iNbNj  */
									/*    b1:PLLE = 0 : PLL֎~iRCUj      */
									/*    b0:PLOCK = 0 : PLLŒ艻oiǂݍ݂̂݁j */
	OCR1A = TMR1_10mS/2;			/*  1`TMR1_10mSȂ炢ł */
	OCR1B = 0x00;					/*  gp                         */
	OCR1C = TMR1_10mS;				/*  10mS𐶐l           */
}

/* OCR1ARyA}b`荞ݏ  *******************************************/
/*  : 荞݂󂯕t TIFR.OCF1AtO'0'ɂȂB                */

ISR(TIMER1_COMPA_vect)
{
	if(s_byTmWait10m != 0){
		s_byTmWait10m--;
	}
	//  10mSŎs鏈LqB
	Chat_Cancel();
	Start_AD();
}

/******************************************************************************/
/*                      ^C}OgPWMg`o                           */
/******************************************************************************/

/* ^C}NbN1MHzƂȂTCCR0B.CS0[2-0]̒l */
#if PCLK == CLK01			/* PCLK=1MHẑƂ */
#define CLK_SEL	(0x01)				/* 1/1 = 1MHz */
#elif PCLK == CLK08			/* PCLK=8MHẑƂ */
#define CLK_SEL	(0x02)				/* 1/8 = 1MHz */
#elif
#error Peripheral Clock Setting Error.
#endif

#define UP_LMT_VAL	(250)			/* ̒lȏ͑S_Ƃ */
#define LO_LMT_VAL	(5)				/* ̒lȉ͏Ƃ   */

static	BYTE	s_byValBack1;		/* OC0APWMl̃obNAbv */
static	BYTE	s_byValBack2;		/* OC0BPWMl̃obNAbv */

/* Wrbg^C}Ȍ ***************************************************/
/*  : ̏ŁA[qOC0A,OC0BI/O|[gƂȂ'0'o͂B   */
/*      : ܂AFLEDCgFLEDɏB                              */

static void Init_Timer0(void)
{
	/* I/O|[gȁ */
	PORTB &= 0xFC;					/* 炩PB1,PB0'0'o͂Ă */
	DDRB  |= 0x03;					/* PB1,PB0o̓|[gɂ */
	/* ʃ^C}䃌WX^iӁF^C}Oƃ^C}P̋pj */
	GTCCR &= 0x7E;					/*    b7:TSM   = 0 : ^C}/JE^Ȃ    */
									/*    b6:PWM1B = X : PWMBHH               */
									/* b5-b4:COM1B1`COM1B0 = XX : OC1B[qIHH */
									/*    b3:FOC1B = X : OC1BύXHH           */
									/*    b2:FOC1A = X : OC1AύXHH           */
									/*    b1:PSR1  = X : vXP1̃ZbgHH  */
									/*    b0:PSR0  = 0 : vXP0̃Zbg  */
	/* 䃌WX^` */
	TCCR0A = 0x03;					/* b7-b6:COM0A[1-0] = 00 : [qOC0AI/O|[gɐݒ  */
									/* b5-b4:COM0B[1-0] = 00 : [qOC0BI/O|[gɐݒ  */
									/* b3-b2:irbgȂj                                         */
									/* b1-b0:WGM0[1-0]  = 11 : 8rbgPWM[hiWGM02=0Ƃj*/
	/* 䃌WX^a */
	TCCR0B = CLK_SEL;				/*    b7:FOC0A = 0 : OC0AύXȂ               */
									/*    b6:FOC0B = 0 : OC0BύXȂ               */
									/* b5-b4:irbgȂj                             */
									/*    b3:WGM02 = 0 : 8rbgPWM[hTOP=0xFF */
									/* b2-b0:CS0[2-0] = ??? : 1MHzƂȂliCLK_SELj   */
	/* ^C}OJE^ */
	TCNT0  = 0;						/* ^C}JE^NA */
	/* RyA}b`WX^` */
	OCR0A  = 0;						/* lƂ0x00Zbg  */
	/* RyA}b`WX^a */
	OCR0B  = 0;						/* lƂ0x00Zbg  */
	/* 荞݋WX^iӁF^C}Oƃ^C}P̋pj */
	TIMSK &= 0xE5;					/*    b7:irbgȂj                         */
									/*    b6:OCIE1A = X : OCR1AƈvŊ荞݁HH */
									/*    b5:OCIE1B = X : OCR1BƈvŊ荞݁HH */
									/*    b4:OCIE0A = 0 : OCR0AƈvŊ荞݋֎~ */
									/*    b3:OCIE0B = 0 : OCR0BƈvŊ荞݋֎~ */
									/*    b2:TOIE1  = X : TCNT1I[o[t[荞݁HH */
									/*    b1:TOIE0  = 0 : TCNT0I[o[t[荞݋֎~ */
									/*    b0:irbgȂj                         */
	/* 荞ݗvtOWX^iӁF^C}Oƃ^C}P̋pj */
	TIFR = 0x1A;					/*    b7:irbgȂj                                       */
									/*    b6:OCF1A = 0 :  iOCR1AƈvŊ荞ݗvj       */
									/*    b5:OCF1B = 0 :  iOCR1BƈvŊ荞ݗvj       */
									/*    b4:OCF0A = 1 : OCF0ANAiOCR0AƈvŊ荞ݗvj */
									/*    b3:OCF0B = 1 : OCF0BNAiOCR0BƈvŊ荞ݗvj */
									/*    b2:TOV1  = 0 :  iTCNT1I[o[t[荞ݗvj */
									/*    b1:TOV0  = 1 : TOV0NA iTCNT0I[o[t[荞ݗvj */
									/*    b0:irbgȂj                                       */
	/* pϐ */
//	s_byValBack1 = 0;				/* ftHg 0̃nYȂ̂ŃRgAEg */
//	s_byValBack2 = 0;
}

/* w̒lŒ[qOC0A,OC0BPWMo͂ ************************************/
/*  : byValue1 = OC0A[qiFLEDpPWMójp̏o̓f[^i0`255j    */
/*      : byValue2 = OC0B[qigFLEDpPWMójp̏o̓f[^i0`255j    */
/* ߒl : Ȃ                                                               */

static void Output_PWM(BYTE byValue1, BYTE byValue2)
{
	/* OC0A[qiFLEDpPWMój̐ */
	if(UP_LMT_VAL <= byValue1){			/* z */
		if(s_byValBack1 < UP_LMT_VAL){
//			OCR0A  = byValue1;			/* RyA}b`WX^`Zbg */
			TCCR0A &= 0x3F;				/*  b7-b6:COM0A[1-0] = 00 : [qOC0AI/O|[gɐݒ */
			PORTB |= 0x01;				/* OC0A[q'1'o */
		}
	}
	else if(byValue1 <= LO_LMT_VAL){	/* Ƃ */
		if(LO_LMT_VAL < s_byValBack1){
//			OCR0A  = byValue1;			/* RyA}b`WX^`Zbg */
			TCCR0A &= 0x3F;				/*  b7-b6:COM0A[1-0] = 00 : [qOC0AI/O|[gɐݒ */
			PORTB &= 0xFE;				/* OC0A[q'0'o */
		}
	}
	else{
		if(byValue1 != s_byValBack1){
			OCR0A  = byValue1;			/* RyA}b`WX^`Zbg */
			TCCR0A = (TCCR0A & 0x3F) | 0x80;	/* b7-b6:COM0A[1-0] = 10 : RyA}b``Œ[qOC0A'0'o  */
		}
	}
	s_byValBack1 = byValue1;

	/* OC0B[qigFLEDpPWMój̐ */
	if(UP_LMT_VAL <= byValue2){			/* z */
		if(s_byValBack2 < UP_LMT_VAL){
//			OCR0B  = byValue2;			/* RyA}b`WX^aZbg */
			TCCR0A &= 0xCF;				/*  b5-b4:COM0B[1-0] = 00 : [qOC0BI/O|[gɐݒ */
			PORTB |= 0x02;				/* OC0B[q'1'o */
		}
	}
	else if(byValue2 <= LO_LMT_VAL){	/* Ƃ */
		if(LO_LMT_VAL < s_byValBack2){
//			OCR0B  = byValue2;			/* RyA}b`WX^aZbg */
			TCCR0A &= 0xCF;				/*  b5-b4:COM0B[1-0] = 00 : [qOC0BI/O|[gɐݒ */
			PORTB &= 0xFD;				/* OC0B[q'0'o */
		}
	}
	else{
		if(byValue2 != s_byValBack2){
			OCR0B  = byValue2;			/* RyA}b`WX^aZbg */
			TCCR0A = (TCCR0A & 0xCF) | 0x20;	/* b5-b4:COM0B[1-0] = 10 : RyA}b`aŒ[qOC0B'0'o  */
		}
	}
	s_byValBack2 = byValue2;
}


/******************************************************************************/
/*                    XCb`̃`^OLZ                        */
/******************************************************************************/

/* Ɨpƒ萔 *********************************************************/

#define SWBITPOS	0x08			/* XCb`̈ʒu̓rbgR */
static BYTE	s_bySwProcess;
static BYTE	s_bySwState;
static BYTE	s_bySwValid;

/* I/O|[gab3̏ ****************************************************/

static inline void Init_PortB(void)
{
	PORTB &= 0xF7;					/* PB3̓[hɂ */
//	s_bySwProcess = 0;				/* ftHg0x00̃nYȂ̂ŃRgAEg */
//	s_bySwState   = 0;
//	s_bySwValid   = 0;
}

/* `^OLZ *****************************************************/
/* Ql : Chat_Cancel()10mSŌĂׂ΁A80mS̃`^OLZ */
/*      : ȂB                                                              */

void Chat_Cancel(void)
{
	s_bySwProcess <<= 1;
	if((PINB & SWBITPOS) == 0){		/* XCb`ONĂ */
		s_bySwProcess |= 0x01;		/* ONĂb0='1'Ƃ */
	}
	if(s_bySwProcess == 0x00){		/* Wȏ OFFA */
		s_bySwState = _OFF_;
	}
	else if(s_bySwProcess == 0xFF){	/* Wȏ  ONA */
		if(s_bySwState != _ON_){
			s_bySwState = _ON_;
			s_bySwValid = _ON_;
		}
	}
}

/* XCb`ON/OFFԓǂݍ ***********************************************/
/* @\ : XCb` ONꍇ_ON_ԂAȊOł_OFF_ԂB        */

static BYTE Get_SwCondition(void)
{
	cli();							/* 荞݋֎~ */
	if(s_bySwValid == _ON_){		/* ONĂ */
		s_bySwValid = _OFF_;
 		sei();						/* 荞݋ */
		return _ON_;
	}
 	sei();							/* 荞݋ */
	return _OFF_;
}

/******************************************************************************/
/*                               C֐                                   */
/******************************************************************************/

int main(void)
{
	BYTE	bySwCurr;

	Init_PortB();					/* I/O|[gȁ */
	Init_Timer1();					/* ^C}P̏ */
	Init_Timer0();					/* ^C}Ȍ */
	Init_ADC();						/* A/DRo[^̏ */
 	sei();							/* 荞݋ */
	/* ҂ */
	s_byTmWait10m = 10;				/* 100mS҂ */
	while(s_byTmWait10m != 0);
	
	/* C[v */
	bySwCurr = _OFF_;
	while(1){
		/* XCb`тON/OFF鏈 */
		if(Get_SwCondition() == _ON_){
			if(bySwCurr == _OFF_){
				bySwCurr = _ON_;
			}
			else{
				bySwCurr = _OFF_;
			}
		}
		/* OFFԂ̎ */
		if(bySwCurr == _OFF_){
			Output_PWM(0, 0);
		}
		/* ONԂ̎ */
		else{
			Output_PWM(s_byADdata1, s_byADdata2);
		}
	}
	return 0;
}

/* End of File */
